@@ -1,3 +1,23 @@
+/*
+ * FigTreeFrame.java
+ *
+ * Copyright (C) 2012 Andrew Rambaut
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
+
 package figtree.application;
 
 import com.itextpdf.text.Document;
@@ -6,11 +26,15 @@ import com.itextpdf.text.pdf.DefaultFontMapper;
 import com.itextpdf.text.pdf.PdfContentByte;
 import com.itextpdf.text.pdf.PdfTemplate;
 import com.itextpdf.text.pdf.PdfWriter;
+import figtree.treeviewer.decorators.DiscreteColourDecorator;
+import figtree.treeviewer.decorators.HSBDiscreteColourDecorator;
+import figtree.treeviewer.painters.StatesPainter;
 import jebl.evolution.alignments.Alignment;
 import jebl.evolution.alignments.BasicAlignment;
 import jebl.evolution.graphs.Node;
 import jebl.evolution.io.*;
 import jebl.evolution.taxa.Taxon;
+import jebl.evolution.trees.RootedTree;
 import jebl.evolution.trees.Tree;
 import jebl.util.Attributable;
 import jam.controlpalettes.BasicControlPalette;
@@ -51,12 +75,11 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
 
         setTitle(title);
 
-
         setImportAction(importAction);
         //       setImportAction(importCharactersAction);
         setExportAction(exportTreesAction);
 
-        treeViewer = new ExtendedTreeViewer();
+        treeViewer = new ExtendedTreeViewer(this);
         controlPalette = new BasicControlPalette(200, BasicControlPalette.DisplayMode.ONLY_ONE_OPEN);
 
         figTreePanel = new FigTreePanel(this, treeViewer, controlPalette);
@@ -245,7 +268,15 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
         final ToolbarAction prevTreeToolbarAction =
                 new ToolbarAction(null, "Previous Tree...", prevIcon) {
                     public void actionPerformed(ActionEvent e){
-                        treeViewer.showPreviousTree();
+                        if (treeViewer.isRootingOn() && treeViewer.getRootingType() == TreePane.RootingType.USER_ROOTING) {
+                            JOptionPane.showMessageDialog(FigTreeFrame.this, "Cannot switch trees when user rooting option is on.\n" +
+                                    "Turn this option off to switch trees",
+                                    "Unable to switch trees",
+                                    JOptionPane.ERROR_MESSAGE);
+
+                        } else {
+                            treeViewer.showPreviousTree();
+                        }
                     }
                 };
         JButton prevTreeToolButton = new ToolbarButton(prevTreeToolbarAction, true);
@@ -257,7 +288,15 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
         final ToolbarAction nextTreeToolbarAction =
                 new ToolbarAction(null, "Next Tree...", nextIcon) {
                     public void actionPerformed(ActionEvent e){
-                        treeViewer.showNextTree();
+                        if (treeViewer.isRootingOn() && treeViewer.getRootingType() == TreePane.RootingType.USER_ROOTING) {
+                            JOptionPane.showMessageDialog(FigTreeFrame.this, "Cannot switch trees when user rooting option is on.\n" +
+                                    "Turn this option off to switch trees",
+                                    "Unable to switch trees",
+                                    JOptionPane.ERROR_MESSAGE);
+
+                        } else {
+                            treeViewer.showNextTree();
+                        }
                     }
                 };
         JButton nextTreeToolButton = new ToolbarButton(nextTreeToolbarAction, true);
@@ -882,17 +921,17 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
                 treeViewer.setTaxonAnnotations(annotations);
 
                 // Hack to show tips states...
-//                String[] annotationNames = new String[annotations.keySet().size()];
-//                DiscreteColorDecorator[] decorators = new DiscreteColorDecorator[annotations.keySet().size()];
-//
-//                int i = 0;
-//                for (AnnotationDefinition definition: annotations.keySet()) {
-//                    Map<Taxon, Object> annotation = annotations.get(definition);
-//                    annotationNames[i] = definition.getName();
-//                    decorators[i] = new DiscreteColorDecorator(annotationNames[i], annotation.keySet());
-//                    i++;
-//                }
-//                treeViewer.setTipLabelPainter(new StatesPainter(annotationNames, decorators));
+                String[] annotationNames = new String[annotations.keySet().size()];
+                DiscreteColourDecorator[] decorators = new DiscreteColourDecorator[annotations.keySet().size()];
+
+                int i = 0;
+                for (AnnotationDefinition definition: annotations.keySet()) {
+                    Map<Taxon, Object> annotation = annotations.get(definition);
+                    annotationNames[i] = definition.getName();
+                    decorators[i] = new HSBDiscreteColourDecorator(annotationNames[i], annotation.keySet());
+                    i++;
+                }
+                treeViewer.setTipLabelPainter(new StatesPainter(annotationNames, decorators));
 
 
             } catch (FileNotFoundException fnfe) {
@@ -1045,6 +1084,49 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
         return true;
     }
 
+    private void doImportColourScheme() {
+
+        FileDialog dialog = new FileDialog(this,
+                "Import Colour Scheme...",
+                FileDialog.LOAD);
+
+        dialog.setVisible(true);
+        if (dialog.getFile() != null) {
+            File file = new File(dialog.getDirectory(), dialog.getFile());
+
+            try {
+                importColourSchemeFromFile(file);
+            } catch (FileNotFoundException fnfe) {
+                JOptionPane.showMessageDialog(this, "Unable to open file: File not found",
+                        "Unable to open file",
+                        JOptionPane.ERROR_MESSAGE);
+            } catch (IOException ioe) {
+                JOptionPane.showMessageDialog(this, "Unable to read file: " + ioe.getMessage(),
+                        "Unable to read file",
+                        JOptionPane.ERROR_MESSAGE);
+            }
+        }
+
+    }
+
+    protected boolean importColourSchemeFromFile(File file) throws FileNotFoundException, IOException {
+
+        final String fileName = file.getName();
+        SequenceImporter importer = new NexusImporter(new FileReader(file));
+        try {
+            Alignment alignment = new BasicAlignment(importer.importSequences());
+
+            treeViewer.setCharacters(alignment);
+        } catch (ImportException ie) {
+            JOptionPane.showMessageDialog(this, "Error reading characters file: " + ie.getMessage(),
+                    "Import Error",
+                    JOptionPane.ERROR_MESSAGE);
+            return false;
+        }
+
+        return true;
+    }
+
     public final void doExport() {
 
         if (exportTreeDialog == null) {
@@ -1069,7 +1151,8 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
                             exportTreeDialog.allTrees(),
                             exportTreeDialog.asDisplayed(),
                             exportTreeDialog.includeFigTreeBlock(),
-                            exportTreeDialog.includeAnnotations());
+                            exportTreeDialog.includeAnnotations(),
+                            false);
                     writer.close();
                 } catch (IOException ioe) {
                     JOptionPane.showMessageDialog(this, "Error writing tree file: " + ioe.getMessage(),
@@ -1083,7 +1166,7 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
 
     public final void doExportGraphic() {
         ExportDialog export = new ExportDialog();
-        export.showExportDialog( this, "Export view as ...", treeViewer.getContentPane(), "export" );
+        export.showExportDialog(this, "Export view as ...", treeViewer.getContentPane(), "export");
     }
 
 
@@ -1131,7 +1214,7 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
     public void doCopy() {
         StringWriter writer = new StringWriter();
         try {
-            writeTreeFile(writer, ExportTreeDialog.Format.NEXUS, true, false, false, true);
+            writeTreeFile(writer, ExportTreeDialog.Format.NEXUS, true, false, false, true, true);
         } catch (IOException e) {
             e.printStackTrace();
         }
@@ -1199,7 +1282,8 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
                                  boolean writeAllTrees,
                                  boolean writeAsDisplayed,
                                  boolean writeFigTreeBlock,
-                                 boolean writeAnnotations) throws IOException {
+                                 boolean writeAnnotations,
+                                 boolean writeSelectedSubtree) throws IOException {
 
         Map<String, Object> settings = null;
         if (writeFigTreeBlock) {
@@ -1207,20 +1291,35 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
             controlPalette.getSettings(settings);
         }
 
-        List<Tree> trees = treeViewer.getTreesAsViewed();
+        List<Tree> trees = new ArrayList<Tree>();
 
-        switch (format) {
-            case NEWICK:
-                NewickExporter newickExporter = new NewickExporter(writer);
-                newickExporter.exportTrees(trees);
-                break;
-            case NEXUS:
-                FigTreeNexusExporter nexusExporter = new FigTreeNexusExporter(writer, writeAnnotations);
-                nexusExporter.exportTrees(trees);
-                if (settings != null) {
-                    nexusExporter.writeFigTreeBlock(settings);
-                }
-                break;
+        if (writeSelectedSubtree) {
+            RootedTree tree = treeViewer.getSelectedSubtree();
+            if (tree != null) {
+                trees.add(tree);
+            }
+        } else {
+            trees.addAll(treeViewer.getTreesAsViewed());
+        }
+
+        if (trees.size() > 0) {
+            switch (format) {
+                case NEWICK:
+                    NewickExporter newickExporter = new NewickExporter(writer);
+                    newickExporter.exportTrees(trees);
+                    break;
+                case NEXUS:
+                    FigTreeNexusExporter nexusExporter = new FigTreeNexusExporter(writer, writeAnnotations);
+                    nexusExporter.exportTrees(trees);
+                    if (settings != null) {
+                        nexusExporter.writeFigTreeBlock(settings);
+                    }
+                    break;
+                case JSON:
+                    JSONTreeExporter jsonExporter = new JSONTreeExporter(writer, writeAnnotations);
+                    jsonExporter.exportTrees(trees);
+                    break;
+            }
         }
 
         writer.close();
@@ -1305,6 +1404,16 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
         return treeViewer.getContentPane();
     }
 
+    @Override
+    public Action getImportColourSchemeAction() {
+        return importColourSchemeAction;
+    }
+
+    @Override
+    public Action getExportColourSchemeAction() {
+        return exportColourSchemeAction;
+    }
+
     public Action getExportTreesAction() {
         return exportTreesAction;
     }
@@ -1421,6 +1530,18 @@ public class FigTreeFrame extends DocumentFrame implements FigTreeFileMenuHandle
         }
     };
 
+    private AbstractAction importColourSchemeAction = new AbstractAction("Import Colour Scheme...") {
+        public void actionPerformed(ActionEvent ae) {
+            doImportColourScheme();
+        }
+    };
+
+    private AbstractAction exportColourSchemeAction = new AbstractAction("Export Colour Scheme...") {
+        public void actionPerformed(ActionEvent ae) {
+//            doExportColourScheme();
+        }
+    };
+
     private AbstractAction exportTreesAction = new AbstractAction("Export Trees...") {
         public void actionPerformed(ActionEvent ae) {
             doExport();
