@@ -3,6 +3,7 @@ package figtree.treeviewer;
 import jam.panels.OptionsPanel;
 
 import javax.swing.*;
+import javax.swing.text.JTextComponent;
 import java.awt.*;
 
 /**
@@ -30,6 +31,9 @@ public class ControllerOptionsPanel extends OptionsPanel {
             comp.setFont(font);
         }
         comp.putClientProperty("JComponent.sizeVariant", "small");
+        if (comp instanceof JSpinner && font != null) {
+            ((JSpinner.NumberEditor)((JSpinner)comp).getEditor()).getTextField().setFont(font);
+        }
         if (comp instanceof JButton) {
             comp.putClientProperty("JButton.buttonType", "roundRect");
         }
