@@ -1,22 +1,10 @@
-/*
- * AlignmentPanel.java
- *
- * (c) 2002-2005 BEAST Development Core Team
- *
- * This package may be distributed under the
- * Lesser Gnu Public Licence (LGPL)
- */
-
 package figtree.treeviewer;
 
+import figtree.treeviewer.painters.*;
 import jebl.evolution.graphs.Node;
 import jebl.evolution.taxa.Taxon;
 import jebl.evolution.trees.*;
 import figtree.treeviewer.decorators.Decorator;
-import figtree.treeviewer.painters.LabelPainter;
-import figtree.treeviewer.painters.ScalePainter;
-import figtree.treeviewer.painters.ScaleGridPainter;
-import figtree.treeviewer.painters.NodeBarPainter;
 import figtree.treeviewer.treelayouts.TreeLayout;
 import jam.panels.StatusProvider;
 
@@ -33,13 +21,20 @@ import java.util.regex.PatternSyntaxException;
  * @version $Id: DefaultTreeViewer.java 822 2007-10-26 13:50:26Z rambaut $
  */
 public class DefaultTreeViewer extends TreeViewer {
-    private final static double MAX_ZOOM = 20;
-    private final static double MAX_VERTICAL_EXPANSION = 20;
+    private final static double ZOOM_SCALE = 0.02;
+    private final static double VERTICAL_EXPANSION_SCALE = 0.02;
+    private final static double ZOOM_POWER = 1.2;
+
+    public DefaultTreeViewer() {
+        this(null);
+    }
 
     /**
      * Creates new TreeViewer
      */
-    public DefaultTreeViewer() {
+    public DefaultTreeViewer(JFrame frame) {
+        this.frame = frame;
+
         setLayout(new BorderLayout());
 
         this.treePane = new TreePane();
@@ -62,7 +57,6 @@ public class DefaultTreeViewer extends TreeViewer {
         // This overrides MouseListener and MouseMotionListener to allow selection in the TreePane -
         // It installs itself within the constructor.
         treePaneSelector = new TreePaneSelector(treePane);
-
         treePaneRollOver = new TreePaneRollOver(treePane);
 
         setFocusable(true);
@@ -86,21 +80,25 @@ public class DefaultTreeViewer extends TreeViewer {
     public void addTree(Tree tree) {
         this.trees.add(tree);
 
-        if (treePane.getTipLabelPainter() != null) {
-            treePane.getTipLabelPainter().setupAttributes(trees);
-        }
-
-        if (treePane.getBranchLabelPainter() != null) {
-            treePane.getBranchLabelPainter().setupAttributes(trees);
-        }
-
-        if (treePane.getNodeLabelPainter() != null) {
-            treePane.getNodeLabelPainter().setupAttributes(trees);
-        }
-
-        if (treePane.getNodeBarPainter() != null) {
-            treePane.getNodeBarPainter().setupAttributes(trees);
-        }
+//        if (treePane.getTipLabelPainter() != null) {
+//            treePane.getTipLabelPainter().setupAttributes(trees);
+//        }
+//
+//        if (treePane.getBranchLabelPainter() != null) {
+//            treePane.getBranchLabelPainter().setupAttributes(trees);
+//        }
+//
+//        if (treePane.getNodeLabelPainter() != null) {
+//            treePane.getNodeLabelPainter().setupAttributes(trees);
+//        }
+//
+//        if (treePane.getNodeBarPainter() != null) {
+//            treePane.getNodeBarPainter().setupAttributes(trees);
+//        }
+//
+//        if (treePane.getLegendPainter() != null) {
+//            treePane.getLegendPainter().setupAttributes(trees);
+//        }
     }
 
     public void addTrees(Collection<? extends Tree> trees) {
@@ -127,6 +125,10 @@ public class DefaultTreeViewer extends TreeViewer {
         return treeAsDisplayed;
     }
 
+    public RootedTree getSelectedSubtree() {
+        return treePane.getSelectedSubtree();
+    }
+
     public int getCurrentTreeIndex() {
         return currentTreeIndex;
     }
@@ -141,6 +143,14 @@ public class DefaultTreeViewer extends TreeViewer {
     }
 
     public void showTree(int index) {
+        if (isRootingOn() && getRootingType() == TreePane.RootingType.USER_ROOTING) {
+            JOptionPane.showMessageDialog(frame, "Cannot switch trees when user rooting option is on.\n" +
+                    "Turn this option off to switch trees",
+                    "Unable to switch trees",
+                    JOptionPane.ERROR_MESSAGE);
+            return;
+        }
+
         Tree tree = trees.get(index);
         if (tree instanceof RootedTree) {
             treePane.setTree((RootedTree)tree);
@@ -170,15 +180,20 @@ public class DefaultTreeViewer extends TreeViewer {
     }
 
     private boolean zoomPending = false;
-    private double zoom = 0.0, verticalExpansion = 0.0;
+    private double zoom = 0.0;
+    private double verticalExpansion = 0.0;
 
     public void setZoom(double zoom) {
-        this.zoom = zoom * MAX_ZOOM;
+        double n = Math.max(treePane.getTree().getTaxa().size(), 50);
+        this.zoom = Math.pow(zoom * n * ZOOM_SCALE, ZOOM_POWER);
+//        this.zoom = zoom * MAX_ZOOM;
         refreshZoom();
     }
 
     public void setVerticalExpansion(double verticalExpansion) {
-        this.verticalExpansion = verticalExpansion * MAX_VERTICAL_EXPANSION;
+        double n = Math.max(treePane.getTree().getTaxa().size(), 50);
+        this.verticalExpansion = Math.pow(verticalExpansion * n * VERTICAL_EXPANSION_SCALE, ZOOM_POWER);
+//        this.verticalExpansion = verticalExpansion * MAX_VERTICAL_EXPANSION;
         refreshZoom();
     }
 
@@ -550,26 +565,30 @@ public class DefaultTreeViewer extends TreeViewer {
         treePaneSelector.setDragMode(dragMode);
     }
 
-    // A load of deligated method calls through to treePane (which is now hidden outside the package).
+    // A load of delegated method calls through to treePane (which is now hidden outside the package).
     public void setTipLabelPainter(LabelPainter<Node> tipLabelPainter) {
         treePane.setTipLabelPainter(tipLabelPainter);
-        tipLabelPainter.setupAttributes(trees);
+//        tipLabelPainter.setupAttributes(trees);
 
     }
 
     public void setNodeLabelPainter(LabelPainter<Node> nodeLabelPainter) {
         treePane.setNodeLabelPainter(nodeLabelPainter);
-        nodeLabelPainter.setupAttributes(trees);
+//        nodeLabelPainter.setupAttributes(trees);
     }
 
     public void setNodeBarPainter(NodeBarPainter nodeBarPainter) {
         treePane.setNodeBarPainter(nodeBarPainter);
-        nodeBarPainter.setupAttributes(trees);
+//        nodeBarPainter.setupAttributes(trees);
+    }
+
+    public void setNodeShapePainter(NodeShapePainter nodeShapePainter) {
+        treePane.setNodeShapePainter(nodeShapePainter);
     }
 
     public void setBranchLabelPainter(LabelPainter<Node> branchLabelPainter) {
         treePane.setBranchLabelPainter(branchLabelPainter);
-        branchLabelPainter.setupAttributes(trees);
+//        branchLabelPainter.setupAttributes(trees);
     }
 
     public void addScalePainter(ScalePainter scalePainter) {
@@ -584,8 +603,13 @@ public class DefaultTreeViewer extends TreeViewer {
         treePane.setScaleGridPainter(scaleGridPainter);
     }
 
-    public void setBranchDecorator(Decorator branchDecorator) {
-        treePane.setBranchDecorator(branchDecorator);
+    public void setLegendPainter(LegendPainter legendPainter) {
+        treePane.setLegendPainter(legendPainter);
+//        legendPainter.setupAttributes(trees);
+    }
+
+    public void setBranchDecorator(Decorator branchDecorator, boolean isGradient) {
+        treePane.setBranchDecorator(branchDecorator, isGradient);
     }
 
     public void setBranchColouringDecorator(String branchColouringAttribute, Decorator branchColouringDecorator) {
@@ -595,7 +619,7 @@ public class DefaultTreeViewer extends TreeViewer {
     public void setNodeBackgroundDecorator(Decorator nodeBackgroundDecorator) {
         treePane.setNodeBackgroundDecorator(nodeBackgroundDecorator);
     }
-    
+
     public void setSelectionColor(Color selectionColor) {
         treePane.setSelectionColor(selectionColor);
     }
@@ -707,4 +731,5 @@ public class DefaultTreeViewer extends TreeViewer {
 
     protected JViewport viewport;
 
+    private final JFrame frame;
 }
\ No newline at end of file
