@@ -2,6 +2,7 @@ package figtree.treeviewer;
 
 import jebl.evolution.graphs.Node;
 import jebl.evolution.graphs.Graph;
+import jebl.evolution.io.ImportException;
 import jebl.evolution.taxa.Taxon;
 import jebl.evolution.trees.*;
 import figtree.treeviewer.decorators.*;
@@ -12,10 +13,10 @@ import javax.swing.*;
 import java.awt.*;
 import java.awt.geom.*;
 import java.awt.print.*;
+import java.io.IOException;
 import java.util.*;
 import java.util.List;
 
-
 /**
  * @author Andrew Rambaut
  * @version $Id: TreePane.java 823 2007-10-28 20:28:47Z stevensh $
@@ -84,7 +85,8 @@ public class TreePane extends JComponent implements PainterListener, Printable {
                 Node left = newTree.getParent(rootingNode);
                 if (left != null) {
                     // rooting length should be [0, 1]
-                    double length = newTree.getLength(rootingNode) * rootingLength;
+
+                    double length = newTree.hasLengths() ? newTree.getLength(rootingNode) * rootingLength : 1.0;
                     try {
                         newTree = new ReRootedTree(newTree, left, rootingNode, length);
                     } catch (Graph.NoEdgeException e) {
@@ -172,8 +174,14 @@ public class TreePane extends JComponent implements PainterListener, Printable {
     }
 
     public void setRootLocation(Node node, double length) {
-        if (isRootingOn()) {
-            rootingNode = ((ReRootedTree)tree).getSourceNode(node);
+        RootedTree source = tree;
+
+        if (tree instanceof FilteredRootedTree) {
+            source = ((FilteredRootedTree) tree).getSource();
+        }
+
+        if (source instanceof  ReRootedTree) {
+            rootingNode = ((ReRootedTree)source).getSourceNode(node);
         } else {
             rootingNode = node;
         }
@@ -220,8 +228,9 @@ public class TreePane extends JComponent implements PainterListener, Printable {
 
     }
 
-    public void setBranchDecorator(Decorator branchDecorator) {
+    public void setBranchDecorator(Decorator branchDecorator, boolean isGradient) {
         this.branchDecorator = branchDecorator;
+        this.branchDecoratorGradient = isGradient;
         repaint();
     }
 
@@ -460,81 +469,146 @@ public class TreePane extends JComponent implements PainterListener, Printable {
     public void setSelectedNode(Node selectedNode) {
         selectedNodes.clear();
         selectedTips.clear();
-        addSelectedNode(selectedNode);
+        addSelectedNode(selectedNode, false, false);
     }
 
     public void setSelectedTip(Node selectedTip) {
         selectedNodes.clear();
         selectedTips.clear();
-        addSelectedTip(selectedTip);
+        addSelectedTip(selectedTip, false, false);
     }
 
     public void setSelectedClade(Node selectedNode) {
         selectedNodes.clear();
         selectedTips.clear();
-        addSelectedClade(selectedNode);
+        addSelectedClade(selectedNode, false, false);
     }
 
     public void setSelectedTips(Node selectedNode) {
         selectedNodes.clear();
         selectedTips.clear();
-        addSelectedTips(selectedNode);
+        addSelectedTips(selectedNode, false);
     }
 
     private boolean canSelectNode(Node selectedNode) {
         return selectedNode != null;
     }
+
     public void addSelectedNode(Node selectedNode) {
-        if ( canSelectNode(selectedNode) ) {
-            selectedNodes.add(selectedNode);
-        }
+        addSelectedNode(selectedNode, false, false);
+    }
+
+    public void addSelectedNode(Node selectedNode, boolean toggle, boolean extend) {
+        amendNodeSelection(selectedNode, toggle, extend);
         fireSelectionChanged();
         clearSelectionPaths();
         repaint();
     }
 
     public void addSelectedTip(Node selectedTip) {
-        if (selectedTip != null) {
-            this.selectedNodes
-                    .add(selectedTip);
-        }
+        addSelectedTip(selectedTip, false, false);
+    }
+
+    public void addSelectedTip(Node selectedTip, boolean toggle, boolean extend) {
+        amendNodeSelection(selectedTip, toggle, extend);
         selectTipsFromSelectedNodes();
         fireSelectionChanged();
         clearSelectionPaths();
         repaint();
     }
 
+
     public void addSelectedClade(Node selectedNode) {
-        if ( canSelectNode(selectedNode) ) {
-            addSelectedChildClades(selectedNode);
+        addSelectedClade(selectedNode, false, false);
+    }
+
+    public void addSelectedClade(Node selectedNode, boolean toggle, boolean extend) {
+        if (canSelectNode(selectedNode)) {
+            amendCladeSelection(selectedNode, toggle, extend);
         }
         fireSelectionChanged();
         clearSelectionPaths();
         repaint();
     }
 
-    private void addSelectedChildClades(Node selectedNode) {
-        selectedNodes.add(selectedNode);
-        for (Node child : tree.getChildren(selectedNode)) {
-            addSelectedChildClades(child);
+    private void amendNodeSelection(Node selectedNode, boolean toggle, boolean extend) {
+        if ( !canSelectNode(selectedNode) ) {
+            return;
+        }
+
+        if (extend) {
+            Set<Node> nodeSet = new HashSet<Node>(selectedNodes);
+            nodeSet.add(selectedNode);
+            Node mrca = RootedTreeUtils.getCommonAncestorNode(tree, nodeSet);
+
+            for (Node node : nodeSet) {
+                while (node != null && node != mrca) {
+                    amendNodeSelection(node, false, false);
+                    node = tree.getParent(node);
+                }
+            }
+        } else {
+            if (toggle && selectedNodes.contains(selectedNode)) {
+                selectedNodes.remove(selectedNode);
+            } else {
+                selectedNodes.add(selectedNode);
+            }
         }
     }
 
+    private void amendCladeSelection(Node selectedNode, boolean toggle, boolean extend) {
+        if ( !canSelectNode(selectedNode) ) {
+            return;
+        }
+
+        if (extend) {
+            Set<Node> nodeSet = new HashSet<Node>(selectedNodes);
+            nodeSet.add(selectedNode);
+            Node mrca = RootedTreeUtils.getCommonAncestorNode(tree, nodeSet);
+
+            for (Node node : nodeSet) {
+                while (node != null && node != mrca) {
+                    amendCladeSelection(node, false, false);
+                    node = tree.getParent(node);
+                }
+            }
+        } else {
+            if (toggle && selectedNodes.contains(selectedNode)) {
+                selectedNodes.remove(selectedNode);
+            } else {
+                selectedNodes.add(selectedNode);
+            }
+        }
+        for (Node child : tree.getChildren(selectedNode)) {
+            amendCladeSelection(child, toggle, false);
+        }
+    }
+
+
     public void addSelectedTips(Node selectedNode) {
+        addSelectedTips(selectedNode, false);
+    }
+
+    public void addSelectedTips(Node selectedNode, boolean toggle) {
         if (selectedNode != null) {
-            addSelectedChildTips(selectedNode);
+            addSelectedChildTips(selectedNode, toggle);
         }
         fireSelectionChanged();
         clearSelectionPaths();
         repaint();
     }
 
-    private void addSelectedChildTips(Node selectedNode) {
+    private void addSelectedChildTips(Node selectedNode, boolean toggle) {
         if (tree.isExternal(selectedNode)) {
-            selectedTips.add(selectedNode);
-        }
-        for (Node child : tree.getChildren(selectedNode)) {
-            addSelectedChildTips(child);
+            if (toggle && selectedTips.contains(selectedNode)) {
+                selectedTips.remove(selectedNode);
+            } else {
+                selectedTips.add(selectedNode);
+            }
+        } else {
+            for (Node child : tree.getChildren(selectedNode)) {
+                addSelectedChildTips(child, toggle);
+            }
         }
     }
 
@@ -542,7 +616,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         Set<Node> nodes = new HashSet<Node>(selectedNodes);
         selectedNodes.clear();
         for (Node node : nodes) {
-            addSelectedClade(node);
+            addSelectedClade(node, false, false);
         }
         fireSelectionChanged();
         clearSelectionPaths();
@@ -551,7 +625,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
 
     public void selectTipsFromSelectedNodes() {
         for (Node node : selectedNodes) {
-            addSelectedChildTips(node);
+            addSelectedChildTips(node, false);
         }
         selectedNodes.clear();
         fireSelectionChanged();
@@ -562,7 +636,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
     public void selectNodesFromSelectedTips() {
         if (selectedTips.size() > 0) {
             Node node = RootedTreeUtils.getCommonAncestorNode(tree, selectedTips);
-            addSelectedClade(node);
+            addSelectedClade(node, false, false);
         }
 
         selectedTips.clear();
@@ -749,10 +823,9 @@ public class TreePane extends JComponent implements PainterListener, Printable {
                     calibrated = false;
                     repaint();
                 }
-
-                for (Node child : tree.getChildren(node)) {
-                    clearSelectedHilightedNodes(child);
-                }
+            }
+            for (Node child : tree.getChildren(node)) {
+                clearSelectedHilightedNodes(child);
             }
         }
     }
@@ -901,6 +974,24 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         return nodeBarPainter;
     }
 
+
+    public void setNodeShapePainter(NodeShapePainter nodeShapePainter) {
+        nodeShapePainter.setTreePane(this);
+        if (this.nodeShapePainter != null) {
+            this.nodeShapePainter.removePainterListener(this);
+        }
+        this.nodeShapePainter = nodeShapePainter;
+        if (this.nodeShapePainter != null) {
+            this.nodeShapePainter.addPainterListener(this);
+        }
+        calibrated = false;
+        repaint();
+    }
+
+    public NodeShapePainter getNodeShapePainter() {
+        return nodeShapePainter;
+    }
+
     public void addScalePainter(ScalePainter scalePainter) {
         assert scalePainter != null;
 
@@ -937,6 +1028,20 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         repaint();
     }
 
+    public void setLegendPainter(LegendPainter legendPainter) {
+        legendPainter.setTreePane(this);
+        legendPainter.addPainterListener(this);
+
+        this.legendPainter = legendPainter;
+
+        calibrated = false;
+        repaint();
+    }
+
+    public LegendPainter getLegendPainter() {
+        return legendPainter;
+    }
+
     public void setPreferredSize(Dimension dimension) {
         if (treeLayout.maintainAspectRatio()) {
             super.setPreferredSize(new Dimension(dimension.width, dimension.height));
@@ -1015,6 +1120,78 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         return selectedTips;
     }
 
+    public RootedTree getSelectedSubtree() {
+        if (selectedNodes.size() == 0 && selectedTips.size() == 0) {
+            // nothing selected so return the whole tree
+            return tree;
+        }
+
+        SimpleRootedTree newTree = new SimpleRootedTree();
+
+        getSelectedSubtree(newTree, this.tree.getRootNode(), false);
+
+        if (newTree.getRootNode() == null) {
+            // no tree was constructed, most likely because only one tip was selected
+            return null;
+        }
+
+        return newTree;
+    }
+
+    /**
+     * Returns a new node structure within newTree that contains the subtree subtended by selected
+     * nodes of the current tree.
+     * @param newTree
+     * @param node
+     * @param isSelected
+     * @return
+     */
+    private Node getSelectedSubtree(SimpleRootedTree newTree, Node node, boolean isSelected) {
+        Node newNode;
+
+        if (tree.isExternal(node)) {
+            if (isSelected || selectedNodes.contains(node) || selectedTips.contains(node)) {
+                newNode = newTree.createExternalNode(tree.getTaxon(node));
+                newTree.setHeight(newNode, tree.getHeight(node));
+                for (String key : node.getAttributeNames()) {
+                    newNode.setAttribute(key, node.getAttribute(key));
+                }
+            } else {
+                newNode = null;
+            }
+        } else {
+            List<Node> children = new ArrayList<Node>();
+
+            for (Node child : tree.getChildren(node)) {
+                Node subtree = getSelectedSubtree(newTree, child, isSelected);
+                if (subtree != null) {
+                    children.add(subtree);
+                }
+            }
+
+            if (children.size() == 0) {
+                if (selectedNodes.contains(node)) {
+                    // if this node was selected but none of its children then include the entire
+                    // descendent clade...
+                    newNode = getSelectedSubtree(newTree, node, true);
+                } else {
+                    newNode = null;
+                }
+            } else if (children.size() == 1) {
+                // just one child so pass it up...
+                newNode = children.get(0);
+            } else {
+                newNode = newTree.createInternalNode(children);
+                newTree.setHeight(newNode, tree.getHeight(node));
+                for (String key : node.getAttributeNames()) {
+                    newNode.setAttribute(key, node.getAttribute(key));
+                }
+            }
+        }
+
+        return newNode;
+    }
+
     public Rectangle2D getDragRectangle() {
         return dragRectangle;
     }
@@ -1172,7 +1349,9 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         }
 
-        if (!calibrated) calibrate(g2, width, height);
+        if (!calibrated) {
+            calibrate(g2, width, height);
+        }
 
         // save graphics state which draw changes so that upon exit it can be restored
 
@@ -1181,6 +1360,10 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         final Stroke oldStroke = g2.getStroke();
         final Font oldFont = g2.getFont();
 
+        if (legendPainter != null && legendPainter.isVisible()) {
+            legendPainter.paint(g2, this, Painter.Justification.CENTER, legendBounds);
+        }
+
         // Paint scales
         for (ScalePainter scalePainter : scalePainters) {
 
@@ -1285,7 +1468,6 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
 
             Shape branchPath = treeLayoutCache.getBranchPath(node);
-            Shape nodePath = treeLayoutCache.getNodeArea(node);
 
             if (branchColouring != null) {
                 PathIterator iter = branchPath.getPathIterator(transform);
@@ -1323,7 +1505,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
                 Shape transPath = transform.createTransformedShape(branchPath);
                 Paint paint = Color.BLACK;
                 if (branchDecorator != null) {
-                    if (branchDecorator.isGradient()) {
+                    if (branchDecoratorGradient && branchDecorator.allowsGradient()) {
                         branchDecorator.setItems(node, tree.getParent(node));
                         PathIterator iter = transPath.getPathIterator(null);
                         double[] coords = new double[6];
@@ -1355,6 +1537,15 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
         }
 
+        // Paint node shapes
+        if (nodeShapePainter != null && nodeShapePainter.isVisible()) {
+            for (Node node : nodePoints.keySet()) {
+                Point2D point = nodePoints.get(node);
+                point = transform.transform(point, null);
+                nodeShapePainter.paint(g2, node, point, nodeShapeTransforms.get(node));
+            }
+        }
+
         // Paint tip labels
         if (tipLabelPainter != null && tipLabelPainter.isVisible()) {
 
@@ -1481,20 +1672,22 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
         }
 
-        // bounds on nodeShapes
+        // bounds on node bars
         if (!isTransformBranchesOn() && nodeBarPainter != null && nodeBarPainter.isVisible()) {
             nodeBars.clear();
             // Iterate though the nodes
             for (Node node : tree.getInternalNodes()) {
 
                 Rectangle2D shapeBounds = nodeBarPainter.calibrate(g2, node);
-                treeBounds.add(shapeBounds);
-                nodeBars.put(node, nodeBarPainter.getNodeBar());
+                if (shapeBounds != null) {
+                    treeBounds.add(shapeBounds);
+                    nodeBars.put(node, nodeBarPainter.getNodeBar());
+                }
             }
         }
 
-        // adjust the bounds so that the origin is at 0,0
-        //treeBounds = new Rectangle2D.Double(0.0, 0.0, treeBounds.getWidth(), treeBounds.getHeight());
+        // totalTreeBounds includes all the stuff which is not in a tree scale (like labels and shapes) but in
+        // screen pixel scale. This is added to the treeBounds to make space round the edge.
 
         // add the tree bounds
         final Rectangle2D totalTreeBounds = treeBounds.getBounds2D(); // (YH) same as (Rectangle2D) treeBounds.clone();
@@ -1504,29 +1697,10 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         if (tipLabelPainter != null && tipLabelPainter.isVisible()) {
 
             tipLabelWidth = 0.0;
-
-            // Find the longest taxon label
-            for (Node node : externalNodes) {
-
-                tipLabelPainter.calibrate(g2, node);
-                tipLabelWidth = Math.max(tipLabelWidth, tipLabelPainter.getPreferredWidth());
-            }
-
             final double tipLabelHeight = tipLabelPainter.getPreferredHeight();
 
-            // Iterate though the nodes with tip labels
-            for (Node node : treeLayoutCache.getTipLabelPathMap().keySet()) {
-                Rectangle2D labelBounds = new Rectangle2D.Double(0.0, 0.0, tipLabelWidth, tipLabelHeight);
-
-                // Get the line that represents the path for the taxon label
-                Line2D taxonPath = treeLayoutCache.getTipLabelPath(node);
-
-                // Work out how it is rotated and create a transform that matches that
-                AffineTransform taxonTransform = calculateTransform(null, taxonPath, tipLabelWidth, tipLabelHeight, true);
-
-                // and add the translated bounds to the overall bounds
-                totalTreeBounds.add(taxonTransform.createTransformedShape(labelBounds).getBounds2D());
-            }
+            // put this in a recursive function to allow for collapsed node labels
+            calibrateTipLabels(g2, tree.getRootNode(), tipLabelHeight, totalTreeBounds);
         }
 
         if (nodeLabelPainter != null && nodeLabelPainter.isVisible()) {
@@ -1568,6 +1742,22 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
         }
 
+        // bounds on nodeShapes
+        if (nodeShapePainter != null && nodeShapePainter.isVisible()) {
+            nodePoints.clear();
+            // Iterate though the nodes
+            for (Node node : tree.getNodes()) {
+
+                Rectangle2D shapeBounds = nodeShapePainter.calibrate(g2, node);
+                if (shapeBounds != null) {
+                    totalTreeBounds.add(shapeBounds);
+
+                    // just at the centroid in here as the actual shape will be reconstructed when drawing
+                    nodePoints.put(node, new Point2D.Double(shapeBounds.getCenterX(), shapeBounds.getCenterY()));
+                }
+            }
+        }
+
         totalScaleBounds = new Rectangle2D.Double(0.0, 0.0, 0.0, 0.0);
         double y = totalTreeBounds.getY() + totalTreeBounds.getHeight();
         for (ScalePainter scalePainter : scalePainters) {
@@ -1583,6 +1773,13 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         }
         totalTreeBounds.add(totalScaleBounds);
 
+        if (legendPainter != null && legendPainter.isVisible()) {
+            legendPainter.calibrate(g2, this);
+            final double w2 = legendPainter.getPreferredWidth();
+            legendBounds = new Rectangle2D.Double(-w2, 0, w2, treeBounds.getHeight());
+            totalTreeBounds.add(legendBounds);
+        }
+
         final double availableW = width - insets.left - insets.right;
         final double availableH = height - insets.top - insets.bottom;
 
@@ -1633,7 +1830,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         double yOffset = 0.0;
 
         if (maintainAspectRatio) {
-            // If the tree is layed out in both dimensions then we
+            // If the tree is laid out in both dimensions then we
             // need to find out which axis has the least space and scale
             // the tree to that (to keep the aspect ratio.
 
@@ -1658,8 +1855,8 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             yScale = h / treeBounds.getHeight();
 
             // and set the origin in the top left corner
-            xOffset = - (treeBounds.getX() * xScale);
-            yOffset = - totalTreeBounds.getY();
+            xOffset = - totalTreeBounds.getX() + (treeBounds.getX() * xScale);
+            yOffset = - totalTreeBounds.getY() - (treeBounds.getY() * yScale);
 
             treeScale = xScale;
         }
@@ -1770,8 +1967,6 @@ public class TreePane extends JComponent implements PainterListener, Printable {
                 // Get the line that represents the path for the branch label
                 Line2D labelPath = treeLayoutCache.getBranchLabelPath(node);
 
-                // AR - I don't think we need to recalibrate this here
-                // branchLabelPainter.calibrate(g2, node);
                 final double labelHeight = branchLabelPainter.getPreferredHeight();
                 final double labelWidth = branchLabelPainter.getPreferredWidth();
                 final Rectangle2D labelBounds = new Rectangle2D.Double(0.0, 0.0, labelWidth, labelHeight);
@@ -1800,6 +1995,16 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
         }
 
+        if (nodeShapePainter != null && nodeShapePainter.isVisible()) {
+            nodeShapeTransforms.clear();
+            // Iterate though the nodes
+            for (Node node : nodePoints.keySet()) {
+                Line2D shapePath = getTreeLayoutCache().getNodeShapePath(node);
+                if (shapePath != null) {
+                    nodeShapeTransforms.put(node, calculateTransform(transform, shapePath));
+                }
+            }
+        }
 
         y = height;
         for (ScalePainter scalePainter : scalePainters) {
@@ -1821,12 +2026,44 @@ public class TreePane extends JComponent implements PainterListener, Printable {
             }
         }
 
+        if (legendPainter != null && legendPainter.isVisible()) {
+            legendPainter.calibrate(g2, this);
+            final double w2 = legendPainter.getPreferredWidth();
+            legendBounds = new Rectangle2D.Double(0, 0, w2, treeBounds.getHeight());
+        }
+
         calloutPaths.clear();
         clearSelectionPaths();
 
         calibrated = true;
     }
 
+    private void calibrateTipLabels(final Graphics2D g2, final Node node, final double tipLabelHeight, final Rectangle2D totalTreeBounds) {
+
+        if (tree.isExternal(node) || node.getAttribute(COLLAPSE_ATTRIBUTE_NAME) != null) {
+            tipLabelPainter.calibrate(g2, node);
+            double width = tipLabelPainter.getPreferredWidth();
+            tipLabelWidth = Math.max(tipLabelWidth, width);
+
+            Rectangle2D labelBounds = new Rectangle2D.Double(0.0, 0.0, width, tipLabelHeight);
+
+            // Get the line that represents the path for the taxon label
+            Line2D taxonPath = treeLayoutCache.getTipLabelPath(node);
+
+            if (taxonPath != null) {
+                // Work out how it is rotated and create a transform that matches that
+                AffineTransform taxonTransform = calculateTransform(null, taxonPath, tipLabelWidth, tipLabelHeight, true);
+
+                // and add the translated bounds to the overall bounds
+                totalTreeBounds.add(taxonTransform.createTransformedShape(labelBounds).getBounds2D());
+            }
+        } else {
+            for (Node child : tree.getChildren(node)) {
+                calibrateTipLabels(g2, child, tipLabelHeight, totalTreeBounds);
+            }
+        }
+    }
+
     private AffineTransform calculateTransform(AffineTransform globalTransform, Line2D line,
                                                double width, double height, boolean just) {
         final Point2D origin = line.getP1();
@@ -1860,6 +2097,27 @@ public class TreePane extends JComponent implements PainterListener, Printable {
         return lineTransform;
     }
 
+    private AffineTransform calculateTransform(AffineTransform globalTransform, Line2D line) {
+        final Point2D origin = line.getP1();
+        if (globalTransform != null) {
+            globalTransform.transform(origin, origin);
+        }
+
+        // Work out how it is rotated and create a transform that matches that
+        AffineTransform lineTransform = new AffineTransform();
+
+        final double dy = line.getY2() - line.getY1();
+        // efficency
+        if( dy != 0.0 ) {
+            final double dx = line.getX2() - line.getX1();
+            final double angle = dx != 0.0 ? Math.atan(dy / dx) : 0.0;
+            lineTransform.rotate(angle, origin.getX(), origin.getY());
+        }
+
+        return lineTransform;
+    }
+
+
     // Overridden methods to recalibrate tree when bounds change
     public void setBounds(int x, int y, int width, int height) {
         calibrated = false;
@@ -1918,6 +2176,7 @@ public class TreePane extends JComponent implements PainterListener, Printable {
 
     private Decorator branchDecorator = null;
     private Decorator branchColouringDecorator = null;
+    private boolean branchDecoratorGradient = false;
     private String branchColouringAttribute = null;
 
     private Decorator nodeBackgroundDecorator = null;
@@ -1930,12 +2189,18 @@ public class TreePane extends JComponent implements PainterListener, Printable {
 
     private NodeBarPainter nodeBarPainter = null;
 
+    private NodeShapePainter nodeShapePainter = null;
+
     private List<ScalePainter> scalePainters = new ArrayList<ScalePainter>();
     private Rectangle2D totalScaleBounds = null;
     private Map<ScalePainter, Rectangle2D> scaleBounds = new HashMap<ScalePainter, Rectangle2D>();
 
     private ScaleGridPainter scaleGridPainter = null;
 
+    private LegendPainter legendPainter = null;
+    private Rectangle2D legendBounds = new Rectangle2D.Double();
+
+
     private BasicStroke branchLineStroke = new BasicStroke(1.0F, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);
     private BasicStroke calloutStroke = new BasicStroke(0.5F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1.0f, new float[]{0.5f, 2.0f}, 0.0f);
     private Stroke selectionStroke = new BasicStroke(6.0F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
@@ -1961,6 +2226,8 @@ public class TreePane extends JComponent implements PainterListener, Printable {
     private Map<Node, Painter.Justification> branchLabelJustifications = new HashMap<Node, Painter.Justification>();
 
     private Map<Node, Shape> nodeBars = new HashMap<Node, Shape>();
+    private Map<Node, Point2D> nodePoints = new HashMap<Node, Point2D>();
+    private Map<Node, AffineTransform> nodeShapeTransforms = new HashMap<Node, AffineTransform>();
 
     private Map<Node, Shape> calloutPaths = new HashMap<Node, Shape>();
 
