@@ -269,11 +269,10 @@ public class RectilinearTreeLayout extends AbstractTreeLayout {
 
                 cache.nodeLabelPaths.put(node, nodeLabelPath);
 
-                Line2D nodeBarPath = new Line2D.Double(
+                Line2D nodeShapePath = new Line2D.Double(
                         nodePoint.getX(), ty,
                         nodePoint.getX() - 1.0, ty);
-
-                cache.nodeBarPaths.put(node, nodeBarPath);
+                cache.nodeShapePaths.put(node, nodeShapePath);
             }
         } else {
 
@@ -303,6 +302,11 @@ public class RectilinearTreeLayout extends AbstractTreeLayout {
 
             cache.tipLabelPaths.put(node, tipLabelPath);
 
+            Line2D nodeShapePath = new Line2D.Double(
+                    nodePoint.getX(), ty,
+                    nodePoint.getX() - 1.0, ty);
+            cache.nodeShapePaths.put(node, nodeShapePath);
+
             yPosition += yIncrement;
 
         }
@@ -464,7 +468,7 @@ public class RectilinearTreeLayout extends AbstractTreeLayout {
                 nodePoint.getX(), y0,
                 nodePoint.getX() - 1.0, y0);
 
-        cache.nodeBarPaths.put(node, nodeBarPath);
+        cache.nodeShapePaths.put(node, nodeBarPath);
 
         if (showingCartoonTipLabels) {
             constructCartoonTipLabelPaths(tree, node, maxXPos, new double[] { minYPos }, cache);
@@ -554,7 +558,7 @@ public class RectilinearTreeLayout extends AbstractTreeLayout {
 
         Line2D nodeBarPath = new Line2D.Double(xPosition, ty, xPosition - 1.0, ty);
 
-        cache.nodeBarPaths.put(node, nodeBarPath);
+        cache.nodeShapePaths.put(node, nodeBarPath);
 
         Line2D tipLabelPath;
 
